<?php

namespace CreoCode\ContaoFormSubmissions;
use Contao\DataContainer;
use Contao\Database;

/**
 * Class FormSubmissionsDataResend
 */
class FormSubmissionsDataResend extends \BackendModule
{
	/**
	 * Template
	 * @var string
	 */
	protected $strTemplate = 'form_submissions_data_resend';


	/**
	 * Generate the module
	 */
	protected function compile() {		
		// Find missing labels
		if (htmlspecialchars(\Input::post('resendForm'))) {
			$this->Template->mode = 'post';
			$formId = htmlspecialchars(\Input::post('resendForm'));

			$this->Template->submission = $this->Database->prepare("SELECT * FROM tl_form_submissions WHERE id=? limit 1")
                    ->execute($formId)->fetchAssoc();
			$this->Template->submissionData = $this->Database->prepare("SELECT * FROM tl_form_submissions_data WHERE pid=? limit 1")
                    ->execute($this->Template->submission['id'])->fetchAssoc();

            // Resent Form
			$recipient = htmlspecialchars(\Input::post('recipient'));
			$subject = htmlspecialchars(\Input::post('subject'));

			$arrSubmitted = unserialize($this->Template->submissionData['arrSubmitted']);
			$arrLabels = unserialize($this->Template->submissionData['arrLabels']);
			$arrFields = unserialize($this->Template->submissionData['arrFields']);

			//kontakt@creo-code.at
			if ($arrSubmitted && $arrLabels) {
            	$this->resendForm($arrSubmitted, $arrLabels, $arrFields, $recipient, $subject);
            	$this->Template->success = true;
        	}

		} else if (htmlspecialchars(\Input::get('resendForm'))) {
			$formId = htmlspecialchars(\Input::get('resendForm'));

			$this->Template->mode = 'confirm';
			$this->Template->submission = $this->Database->prepare("SELECT * FROM tl_form_submissions WHERE id=? limit 1")
                    ->execute($formId)->fetchAssoc();

                    /*
			$this->Template->submissionSet = $this->Database->prepare("SELECT * FROM tl_form_submissions_set WHERE id=? limit 1")
                    ->execute($this->Template->submission['pid'])->fetchAssoc();
                    */

            $this->Template->submissionData = $this->Database->prepare("SELECT * FROM tl_form_submissions_data WHERE pid=? limit 1")
                    ->execute($this->Template->submission['id'])->fetchAssoc();
		}
	}

	/**
	 * Resend form / look at contaos core function
	 *
	 * @param array $arrSubmitted
	 * @param array $arrLabels
	 * @param array $arrFields
	 */
	protected function resendForm($arrSubmitted, $arrLabels, $arrFields, $recipient, $subject)
	{
		$this->sendViaEmail = true;
		$this->skipEmpty = false;
		$this->format = 'raw';
		$this->recipient = $recipient;

		// Send form data via e-mail
		if ($this->sendViaEmail)
		{
			$keys = array();
			$values = array();
			$fields = array();
			$message = '';
			foreach ($arrSubmitted as $k=>$v)
			{
				if ($k == 'cc')
				{
					continue;
				}
				$v = \StringUtil::deserialize($v);
				// Skip empty fields
				if ($this->skipEmpty && !\is_array($v) && !\strlen($v))
				{
					continue;
				}
				// Add field to message
				$message .= ($arrLabels[$k] ?? ucfirst($k)) . ': ' . (\is_array($v) ? implode(', ', $v) : $v) . "\n";
				// Prepare XML file
				if ($this->format == 'xml')
				{
					$fields[] = array
					(
						'name' => $k,
						'values' => (\is_array($v) ? $v : array($v))
					);
				}
				// Prepare CSV file
				if ($this->format == 'csv')
				{
					$keys[] = $k;
					$values[] = (\is_array($v) ? implode(',', $v) : $v);
				}				
			}
			$recipients = \StringUtil::splitCsv($this->recipient);
			// Format recipients
			foreach ($recipients as $k=>$v)
			{
				$recipients[$k] = str_replace(array('[', ']', '"'), array('<', '>', ''), $v);
			}
			$email = new \Email();
			$email->subject = $subject;
			// Get subject and message
			if ($this->format == 'email')
			{
				$message = $arrSubmitted['message'];
				$email->subject = $arrSubmitted['subject'];
			}
			// Set the admin e-mail as "from" address
			$email->from = $GLOBALS['TL_ADMIN_EMAIL'];
			$email->fromName = $GLOBALS['TL_ADMIN_NAME'];
			/*
			// Get the "reply to" address
			if (!empty(\Input::post('email', true)))
			{
				$replyTo = \Input::post('email', true);
				// Add the name
				if (!empty(\Input::post('name')))
				{
					$replyTo = '"' . \Input::post('name') . '" <' . $replyTo . '>';
				}
				elseif (!empty(\Input::post('firstname')) && !empty(\Input::post('lastname')))
				{
					$replyTo = '"' . \Input::post('firstname') . ' ' . \Input::post('lastname') . '" <' . $replyTo . '>';
				}
				$email->replyTo($replyTo);
			}
			*/
			// Fallback to default subject
			if (!$email->subject)
			{
				//$email->subject = $this->replaceInsertTags($this->subject, false);
				$email->subject = 'No subject';
			}
			// Send copy to sender
			if (!empty($arrSubmitted['cc']))
			{
				$email->sendCc(\Input::post('email', true));
				unset($_SESSION['FORM_DATA']['cc']);
			}
			// Attach XML file
			if ($this->format == 'xml')
			{
				$objTemplate = new FrontendTemplate('form_xml');
				$objTemplate->fields = $fields;
				$objTemplate->charset = Config::get('characterSet');
				$email->attachFileFromString($objTemplate->parse(), 'form.xml', 'application/xml');
			}
			// Attach CSV file
			if ($this->format == 'csv')
			{
				$email->attachFileFromString(\StringUtil::decodeEntities('"' . implode('";"', $keys) . '"' . "\n" . '"' . implode('";"', $values) . '"'), 'form.csv', 'text/comma-separated-values');
			}
			$uploaded = '';
			// Attach uploaded files
			if (!empty($_SESSION['FILES']))
			{
				foreach ($_SESSION['FILES'] as $file)
				{
					// Add a link to the uploaded file
					if ($file['uploaded'])
					{
						$uploaded .= "\n" . Environment::get('base') . \StringUtil::stripRootDir(\dirname($file['tmp_name'])) . '/' . rawurlencode($file['name']);
						continue;
					}
					$email->attachFileFromString(file_get_contents($file['tmp_name']), $file['name'], $file['type']);
				}
			}
			$uploaded = \strlen(trim($uploaded)) ? "\n\n---\n" . $uploaded : '';
			$email->text = \StringUtil::decodeEntities(trim($message)) . $uploaded . "\n\n";
			// Send the e-mail
			try
			{
				$email->sendTo($recipients);
			}
			catch (\Swift_SwiftException $e)
			{
				$this->log('Form "' . $this->title . '" could not be sent: ' . $e->getMessage(), __METHOD__, TL_ERROR);
			}
		}

		// Add a log entry
		$this->log('Form "' . $this->title . '" has been submitted throught the backend from the form submission module.', __METHOD__, TL_FORMS);

	}
}
